@@ -47,7 +47,10 @@ struct _http_proxy
     uint8_t state;
     int proxy_port;
     char *proxy_host;
-    
+    char *user;
+    char *password;
+    int l_user;
+    int l_password;
     char *target_host;
     int target_port;
     char buf[600];
@@ -75,6 +78,9 @@ typedef struct _swClient
     uint32_t http2 :1;
     uint32_t sleep :1;
     uint32_t wait_dns :1;
+    uint32_t shutdow_rw :1;
+    uint32_t shutdown_read :1;
+    uint32_t shutdown_write :1;
 
     /**
      * one package: length check
@@ -98,6 +104,11 @@ typedef struct _swClient
     double timeout;
     swTimer_node *timer;
 
+    /**
+     * signal interruption
+     */
+    double interrupt_time;
+
     /**
      * sendto, read only.
      */
@@ -109,6 +120,12 @@ typedef struct _swClient
     swSocketAddress remote_addr;
 
     swConnection *socket;
+
+    /**
+     * reactor
+     */
+    swReactor *reactor;
+
     void *object;
 
     swString *buffer;
@@ -144,6 +161,7 @@ typedef struct _swClient
 int swClient_create(swClient *cli, int type, int async);
 int swClient_sleep(swClient *cli);
 int swClient_wakeup(swClient *cli);
+int swClient_shutdown(swClient *cli, int __how);
 #ifdef SW_USE_OPENSSL
 int swClient_enable_ssl_encrypt(swClient *cli);
 int swClient_ssl_handshake(swClient *cli);
@@ -164,6 +182,23 @@ typedef struct
 int swDNSResolver_request(char *domain, void (*callback)(char *, swDNSResolver_result *, void *), void *data);
 int swDNSResolver_free();
 
+//----------------------------------------Stream---------------------------------------
+typedef struct _swStream
+{
+    swString *buffer;
+    uint32_t session_id;
+    uint8_t cancel;
+    void (*response)(struct _swStream *stream, char *data, uint32_t length);
+    swClient client;
+} swStream;
+
+swStream* swStream_new(char *dst_host, int dst_port, int type);
+int swStream_send(swStream *stream, char *data, size_t length);
+void swStream_set_protocol(swProtocol *protocol);
+void swStream_set_max_length(swStream *stream, uint32_t max_length);
+int swStream_recv_blocking(int fd, void *__buf, size_t __len);
+//----------------------------------------Stream End------------------------------------
+
 #ifdef __cplusplus
 }
 #endif
