@@ -33,6 +33,7 @@ enum swErrorCode
     SW_ERROR_FILE_EMPTY,
     SW_ERROR_DNSLOOKUP_DUPLICATE_REQUEST,
     SW_ERROR_DNSLOOKUP_RESOLVE_FAILED,
+    SW_ERROR_BAD_IPV6_ADDRESS,
 
     /**
      * connection error
@@ -89,6 +90,7 @@ enum swErrorCode
     SW_ERROR_SERVER_INVALID_LISTEN_PORT,
     SW_ERROR_SERVER_TOO_MANY_LISTEN_PORT,
     SW_ERROR_SERVER_PIPE_BUFFER_FULL,
+    SW_ERROR_SERVER_NO_IDLE_WORKER,
 
     SW_ERROR_SERVER_ONLY_START_ONE,
 
