@@ -183,14 +183,14 @@ typedef unsigned long ulong_t;
 static sw_inline char* swoole_strdup(const char *s)
 {
     size_t l = strlen(s) + 1;
-    char *p = sw_malloc(l);
+    char *p = (char *)sw_malloc(l);
     memcpy(p, s, l);
     return p;
 }
 
 static sw_inline char* swoole_strndup(const char *s, size_t n)
 {
-    char *p = sw_malloc(n + 1);
+    char *p = (char *)sw_malloc(n + 1);
     strncpy(p, s, n);
     p[n] = '\0';
     return p;
@@ -232,6 +232,7 @@ enum swFd_type
     SW_FD_ERROR           = 3, //socket error
     SW_FD_UDP             = 4, //udp socket
     SW_FD_PIPE            = 5, //pipe
+    SW_FD_STREAM          = 6, //stream socket
     SW_FD_WRITE           = 7, //fd can write
     SW_FD_TIMER           = 8, //timer fd
     SW_FD_AIO             = 9, //linux native aio
@@ -297,12 +298,13 @@ enum swLog_level
 //-------------------------------------------------------------------------------
 enum swFactory_dispatch_mode
 {
-    SW_DISPATCH_ROUND = 1,
-    SW_DISPATCH_FDMOD = 2,
-    SW_DISPATCH_QUEUE = 3,
-    SW_DISPATCH_IPMOD = 4,
-    SW_DISPATCH_UIDMOD = 5,
+    SW_DISPATCH_ROUND    = 1,
+    SW_DISPATCH_FDMOD    = 2,
+    SW_DISPATCH_QUEUE    = 3,
+    SW_DISPATCH_IPMOD    = 4,
+    SW_DISPATCH_UIDMOD   = 5,
     SW_DISPATCH_USERFUNC = 6,
+    SW_DISPATCH_STREAM   = 7,
 };
 
 enum swWorker_status
@@ -350,11 +352,9 @@ write(SwooleG.debug_fd, sw_error, __debug_log_n);
 #endif
 
 #ifdef SW_DEBUG
-#define swTrace(str,...)       {printf("[%s:%d@%s]" str "\n",__FILE__,__LINE__,__func__,##__VA_ARGS__);}
-//#define swWarn(str,...)        {printf("[%s:%d@%s]"str"\n",__FILE__,__LINE__,__func__,##__VA_ARGS__);}
+#define swTrace(str,...) if (SwooleG.debug) {printf("[%s:%d@%s]" str "\n",__FILE__,__LINE__,__func__,##__VA_ARGS__);}
 #else
 #define swTrace(str,...)
-//#define swWarn(str,...)        {printf(sw_error);}
 #endif
 
 enum swTraceType
@@ -479,44 +479,41 @@ typedef struct _swConnection
      * is active
      * system fd must be 0. en: timerfd, signalfd, listen socket
      */
-    uint32_t active :1;
-    uint32_t connect_notify :1;
-    uint32_t direct_send :1;
-    uint32_t ssl_send :1;
-    uint32_t reserved_1 :4;
+    uint8_t active;
+    uint8_t connect_notify;
+    uint8_t direct_send;
+    uint8_t ssl_send;
     //--------------------------------------------------------------
-    uint32_t listen_wait :1;
-    uint32_t recv_wait :1;
-    uint32_t send_wait :1;
-    uint32_t close_wait :1;
-    uint32_t overflow :1;
-    uint32_t high_watermark :1;
-    uint32_t removed :1;
-    uint32_t tcp_nopush :1;
+    uint8_t listen_wait;
+    uint8_t recv_wait;
+    uint8_t send_wait;
+    uint8_t close_wait;
+    uint8_t overflow;
+    uint8_t high_watermark;
+    uint8_t removed;
+    uint8_t tcp_nopush;
     //--------------------------------------------------------------
-    uint32_t tcp_nodelay :1;
-    uint32_t ssl_want_read :1;
-    uint32_t ssl_want_write :1;
-    uint32_t http_upgrade :1;
-    uint32_t http2_stream :1;
-    uint32_t reserved_2 :3;
+    uint8_t tcp_nodelay;
+    uint8_t ssl_want_read;
+    uint8_t ssl_want_write;
+    uint8_t http_upgrade;
+    uint8_t http2_stream;
     //--------------------------------------------------------------
     /**
      * server is actively close the connection
      */
-    uint32_t close_actively :1;
-    uint32_t closed :1;
-    uint32_t closing :1;
-    uint32_t close_reset :1;
+    uint8_t close_actively;
+    uint8_t closed;
+    uint8_t closing;
+    uint8_t close_reset;
     /**
      * protected connection, cannot be closed by heartbeat thread.
      */
-    uint32_t protect :1;
-    uint32_t reserved_3 :3;
+    uint8_t protect;
+    uint8_t nonblock;
     //--------------------------------------------------------------
-    uint32_t close_notify :1;
-    uint32_t close_force :1;
-    uint32_t reserved_4 :6;
+    uint8_t close_notify;
+    uint8_t close_force;
     //--------------------------------------------------------------
     /**
      * ReactorThread id
@@ -799,6 +796,8 @@ enum _swPipe_close_which
 {
     SW_PIPE_CLOSE_MASTER = 1,
     SW_PIPE_CLOSE_WORKER = 2,
+    SW_PIPE_CLOSE_READ   = 3,
+    SW_PIPE_CLOSE_WRITE  = 4,
     SW_PIPE_CLOSE_BOTH   = 0,
 };
 
@@ -863,6 +862,18 @@ enum swDNSLookup_cache_type
     SW_DNS_LOOKUP_RANDOM  = (1u << 11),
 };
 
+typedef struct
+{
+    char *hostname;
+    char *service;
+    int family;
+    int socktype;
+    int protocol;
+    int error;
+    void *result;
+    int count;
+} swRequest_getaddrinfo;
+
 //文件锁
 typedef struct _swFileLock
 {
@@ -1244,9 +1255,12 @@ void swoole_print_trace(void);
 void swoole_ioctl_set_block(int sock, int nonblock);
 void swoole_fcntl_set_option(int sock, int nonblock, int cloexec);
 int swoole_gethostbyname(int type, char *name, char *addr);
+int swoole_getaddrinfo(swRequest_getaddrinfo *req);
+char* swoole_string_format(size_t n, const char *format, ...);
 //----------------------core function---------------------
 int swSocket_set_timeout(int sock, double timeout);
-
+int swSocket_create_server(int type, char *address, int port, int backlog);
+//----------------------------------------Socket---------------------------------------
 static sw_inline int swSocket_is_dgram(uint8_t type)
 {
     return (type == SW_SOCK_UDP || type == SW_SOCK_UDP6 || type == SW_SOCK_UNIX_DGRAM);
@@ -1317,6 +1331,7 @@ int swSocket_udp_sendto(int server_sock, char *dst_ip, int dst_port, char *data,
 int swSocket_udp_sendto6(int server_sock, char *dst_ip, int dst_port, char *data, uint32_t len);
 int swSocket_sendfile_sync(int sock, char *filename, off_t offset, size_t length, double timeout);
 int swSocket_write_blocking(int __fd, void *__data, int __len);
+int swSocket_recv_blocking(int fd, void *__data, size_t __len, int flags);
 
 static sw_inline int swWaitpid(pid_t __pid, int *__stat_loc, int __options)
 {
@@ -1453,6 +1468,7 @@ struct _swReactor
     void (*onFinish)(swReactor *);
 
     void (*enable_accept)(swReactor *);
+    int (*can_exit)(swReactor *);
 
     int (*write)(swReactor *, int, void *, int);
     int (*close)(swReactor *, int);
@@ -1506,12 +1522,18 @@ struct _swWorker
     uint8_t deleted;
     uint8_t child_process;
 
+    uint8_t traced;
+    void (*tracer)(struct _swWorker *);
+
     /**
      * tasking num
      */
     sw_atomic_t tasking_num;
 
     time_t start_time;
+    time_t request_time;
+
+    long request_count;
 
 	/**
 	 * worker id
@@ -1532,6 +1554,13 @@ struct _swWorker
 	void *ptr2;
 };
 
+typedef struct
+{
+    int socket;
+    int last_connection;
+    char *socket_file;
+} swStreamInfo;
+
 struct _swProcessPool
 {
     /**
@@ -1540,6 +1569,7 @@ struct _swProcessPool
     uint8_t reloading;
     uint8_t reload_flag;
     uint8_t dispatch_mode;
+    uint8_t ipc_mode;
 
     /**
      * process type
@@ -1555,6 +1585,12 @@ struct _swProcessPool
      * use message queue IPC
      */
     uint8_t use_msgqueue;
+
+    /**
+     * use stream socket IPC
+     */
+    uint8_t use_socket;
+
     /**
      * message queue key
      */
@@ -1569,7 +1605,7 @@ struct _swProcessPool
     void (*onWorkerStop)(struct _swProcessPool *pool, int worker_id);
 
     int (*main_loop)(struct _swProcessPool *pool, swWorker *worker);
-    int (*onWorkerNotFound)(struct _swProcessPool *pool, pid_t pid);
+    int (*onWorkerNotFound)(struct _swProcessPool *pool, pid_t pid, int status);
 
     sw_atomic_t round_id;
     sw_atomic_t run_worker_num;
@@ -1579,6 +1615,7 @@ struct _swProcessPool
     swHashMap *map;
     swReactor *reactor;
     swMsgQueue *queue;
+    swStreamInfo *stream;
 
     void *ptr;
     void *ptr2;
@@ -1743,7 +1780,8 @@ int swReactorKqueue_create(swReactor *reactor, int max_event_num);
 int swReactorSelect_create(swReactor *reactor);
 
 /*----------------------------Process Pool-------------------------------*/
-int swProcessPool_create(swProcessPool *pool, int worker_num, int max_request, key_t msgqueue_key, int ipc_type);
+int swProcessPool_create(swProcessPool *pool, int worker_num, int max_request, key_t msgqueue_key, int ipc_mode);
+int swProcessPool_create_stream_socket(swProcessPool *pool, char *socket_file, int blacklog);
 int swProcessPool_wait(swProcessPool *pool);
 int swProcessPool_start(swProcessPool *pool);
 void swProcessPool_shutdown(swProcessPool *pool);
@@ -1774,15 +1812,24 @@ typedef struct _swChannel
     char head_tag;
     char tail_tag;
     int num;
+    int max_num;
+    /**
+     * Data length, excluding structure
+     */
     size_t bytes;
     int flag;
     int maxlen;
-    void *mem;   //内存块
+    /**
+     * memory point
+     */
+    void *mem;
     swLock lock;
     swPipe notify_fd;
 } swChannel;
 
 swChannel* swChannel_new(size_t size, int maxlen, int flag);
+#define swChannel_empty(ch) (ch->num == 0)
+#define swChannel_full(ch) ((ch->head == ch->tail && ch->tail_tag != ch->head_tag) || (ch->bytes + sizeof(int) * ch->num == ch->size))
 int swChannel_pop(swChannel *object, void *out, int buffer_length);
 int swChannel_push(swChannel *object, void *in, int data_length);
 int swChannel_out(swChannel *object, void *out, int buffer_length);
@@ -1790,6 +1837,7 @@ int swChannel_in(swChannel *object, void *in, int data_length);
 int swChannel_wait(swChannel *object);
 int swChannel_notify(swChannel *object);
 void swChannel_free(swChannel *object);
+void swChannel_print(swChannel *);
 
 swLinkedList* swLinkedList_new(uint8_t type, swDestructor dtor);
 int swLinkedList_append(swLinkedList *ll, void *data);
@@ -1965,7 +2013,6 @@ typedef struct
     uint32_t shutdown :1;
     uint32_t wait_exit :1;
 
-    long request_count;
     int max_request;
 
 #ifdef SW_COROUTINE
@@ -1987,6 +2034,7 @@ typedef struct
     uint8_t factory_lock_target;
     int16_t factory_target_worker;
     swString **buffer_input;
+    swReactor *reactor;
 } swThreadG;
 
 typedef struct
@@ -2013,6 +2061,7 @@ typedef struct
     uint8_t socket_dontwait :1;
     uint8_t dns_lookup_random :1;
     uint8_t use_async_resolver :1;
+    uint8_t debug :1;
 
 
     /**
