@@ -50,7 +50,7 @@
 #include "Client.h"
 #include "async.h"
 
-#define PHP_SWOOLE_VERSION  "2.0.12"
+#define PHP_SWOOLE_VERSION  "2.1.0"
 #define PHP_SWOOLE_CHECK_CALLBACK
 #define PHP_SWOOLE_ENABLE_FASTCALL
 
@@ -143,8 +143,8 @@ extern swoole_object_array swoole_objects;
 #endif
 #endif
 
-#if PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 5
-#error "require PHP version 5.5 or later."
+#if PHP_MAJOR_VERSION < 7
+#error "require PHP version 7.0 or later."
 #endif
 
 #include "php7_wrapper.h"
@@ -224,7 +224,7 @@ enum php_swoole_fd_type
 };
 //---------------------------------------------------------
 #define php_swoole_socktype(type)           (type & (~SW_FLAG_SYNC) & (~SW_FLAG_ASYNC) & (~SW_FLAG_KEEP) & (~SW_SOCK_SSL))
-#define php_swoole_array_length(array)      (Z_ARRVAL_P(array)->nNumOfElements)
+#define php_swoole_array_length(array)      zend_hash_num_elements(Z_ARRVAL_P(array))
 
 #define SW_LONG_CONNECTION_KEY_LEN          64
 
@@ -255,6 +255,7 @@ PHP_FUNCTION(swoole_set_process_name);
 PHP_FUNCTION(swoole_get_local_ip);
 PHP_FUNCTION(swoole_get_local_mac);
 PHP_FUNCTION(swoole_unsupport_serialize);
+PHP_FUNCTION(swoole_coroutine_create);
 
 //---------------------------------------------------------
 //                  swoole_server
@@ -304,6 +305,7 @@ PHP_METHOD(swoole_connection_iterator, offsetExists);
 PHP_METHOD(swoole_connection_iterator, offsetGet);
 PHP_METHOD(swoole_connection_iterator, offsetSet);
 PHP_METHOD(swoole_connection_iterator, offsetUnset);
+PHP_METHOD(swoole_connection_iterator, __destruct);
 #endif
 
 #ifdef SWOOLE_SOCKETS_SUPPORT
@@ -378,13 +380,18 @@ void swoole_process_init(int module_number TSRMLS_DC);
 void swoole_http_server_init(int module_number TSRMLS_DC);
 #ifdef SW_USE_HTTP2
 void swoole_http2_client_init(int module_number TSRMLS_DC);
+#ifdef SW_COROUTINE
+void swoole_http2_client_coro_init(int module_number TSRMLS_DC);
+#endif
 #endif
 void swoole_websocket_init(int module_number TSRMLS_DC);
 void swoole_buffer_init(int module_number TSRMLS_DC);
 void swoole_mysql_init(int module_number TSRMLS_DC);
 void swoole_mmap_init(int module_number TSRMLS_DC);
 void swoole_channel_init(int module_number TSRMLS_DC);
+void swoole_ringqueue_init(int module_number TSRMLS_DC);
 #if PHP_MAJOR_VERSION == 7
+void swoole_channel_coro_init(int module_number TSRMLS_DC);
 void swoole_serialize_init(int module_number TSRMLS_DC);
 #endif
 
@@ -399,6 +406,7 @@ void php_swoole_check_timer(int interval);
 long php_swoole_add_timer(int ms, zval *callback, zval *param, int persistent TSRMLS_DC);
 void php_swoole_clear_all_timer();
 void php_swoole_register_callback(swServer *serv);
+void php_swoole_trace_check(swServer *serv);
 void php_swoole_client_free(zval *object, swClient *cli TSRMLS_DC);
 swClient* php_swoole_client_new(zval *object, char *host, int host_len, int port);
 void php_swoole_client_check_setting(swClient *cli, zval *zset TSRMLS_DC);
@@ -457,6 +465,10 @@ PHPAPI zend_string* php_swoole_serialize(zval *zvalue);
 PHPAPI int php_swoole_unserialize(void *buffer, size_t len, zval *return_value, zval *object_args, long flag);
 #endif
 
+#ifdef SW_COROUTINE
+int php_coroutine_reactor_can_exit(swReactor *reactor);
+#endif
+
 static sw_inline zval* php_swoole_server_get_callback(swServer *serv, int server_fd, int event_type)
 {
     swListenPort *port = (swListenPort *) serv->connection_list[server_fd].object;
@@ -550,6 +562,7 @@ ZEND_BEGIN_MODULE_GLOBALS(swoole)
     zend_bool display_errors;
     zend_bool cli;
     zend_bool use_namespace;
+    zend_bool use_shortname;
     zend_bool fast_serialize;
     long socket_buffer_size;
 ZEND_END_MODULE_GLOBALS(swoole)
@@ -562,6 +575,7 @@ extern ZEND_DECLARE_MODULE_GLOBALS(swoole);
 #define SWOOLE_G(v) (swoole_globals.v)
 #endif
 
+#define SWOOLE_DEFINE(constant)    REGISTER_LONG_CONSTANT("SWOOLE_"#constant, SW_##constant, CONST_CS | CONST_PERSISTENT)
 
 #define SWOOLE_INIT_CLASS_ENTRY(ce, name, name_ns, methods) \
     if (SWOOLE_G(use_namespace)) { \
