@@ -17,6 +17,7 @@
 #include "swoole.h"
 #include "atomic.h"
 
+#include <stdarg.h>
 #include <sys/stat.h>
 #include <sys/resource.h>
 #include <sys/ioctl.h>
@@ -119,6 +120,10 @@ void swoole_init(void)
 
     SwooleG.use_timer_pipe = 1;
 
+#ifdef SW_DEBUG
+    SwooleG.debug = 1;
+#endif
+
     SwooleStats = SwooleG.memory_pool->alloc(SwooleG.memory_pool, sizeof(swServerStats));
     if (SwooleStats == NULL)
     {
@@ -334,8 +339,16 @@ int swoole_sync_writefile(int fd, void *data, int len)
             count -= n;
             written += n;
         }
+        else if (n == 0)
+        {
+            break;
+        }
         else
         {
+            if (errno == EINTR || errno == EAGAIN)
+            {
+                continue;
+            }
             swWarn("write() failed. Error: %s[%d]", strerror(errno), errno);
             break;
         }
@@ -674,6 +687,10 @@ int swoole_sync_readfile(int fd, void *buf, int len)
         }
         else
         {
+            if (errno == EINTR || errno == EAGAIN)
+            {
+                continue;
+            }
             swWarn("read() failed. Error: %s[%d]", strerror(errno), errno);
             break;
         }
@@ -749,67 +766,74 @@ void swoole_ioctl_set_block(int sock, int nonblock)
 void swoole_fcntl_set_option(int sock, int nonblock, int cloexec)
 {
     int opts, ret;
-    do
-    {
-        opts = fcntl(sock, F_GETFL);
-    }
-    while (opts < 0 && errno == EINTR);
 
-    if (opts < 0)
+    if (nonblock >= 0)
     {
-        swSysError("fcntl(%d, GETFL) failed.", sock);
-    }
+        do
+        {
+            opts = fcntl(sock, F_GETFL);
+        }
+        while (opts < 0 && errno == EINTR);
 
-    if (nonblock)
-    {
-        opts = opts | O_NONBLOCK;
-    }
-    else
-    {
-        opts = opts & ~O_NONBLOCK;
-    }
+        if (opts < 0)
+        {
+            swSysError("fcntl(%d, GETFL) failed.", sock);
+        }
 
-    do
-    {
-        ret = fcntl(sock, F_SETFL, opts);
-    }
-    while (ret < 0 && errno == EINTR);
+        if (nonblock)
+        {
+            opts = opts | O_NONBLOCK;
+        }
+        else
+        {
+            opts = opts & ~O_NONBLOCK;
+        }
 
-    if (ret < 0)
-    {
-        swSysError("fcntl(%d, SETFL, opts) failed.", sock);
+        do
+        {
+            ret = fcntl(sock, F_SETFL, opts);
+        }
+        while (ret < 0 && errno == EINTR);
+
+        if (ret < 0)
+        {
+            swSysError("fcntl(%d, SETFL, opts) failed.", sock);
+        }
     }
 
 #ifdef FD_CLOEXEC
-    do
+    if (cloexec >= 0)
     {
-        opts = fcntl(sock, F_GETFD);
-    }
-    while (opts < 0 && errno == EINTR);
+        do
+        {
+            opts = fcntl(sock, F_GETFD);
+        }
+        while (opts < 0 && errno == EINTR);
 
-    if (opts < 0)
-    {
-        swSysError("fcntl(%d, GETFL) failed.", sock);
-    }
+        if (opts < 0)
+        {
+            swSysError("fcntl(%d, GETFL) failed.", sock);
+        }
 
-    if (cloexec)
-    {
-        opts = opts | FD_CLOEXEC;
-    }
-    else
-    {
-        opts = opts & ~FD_CLOEXEC;
-    }
+        if (cloexec)
+        {
+            opts = opts | FD_CLOEXEC;
+        }
+        else
+        {
+            opts = opts & ~FD_CLOEXEC;
+        }
 
-    do
-    {
-        ret = fcntl(sock, F_SETFD, opts);
-    }
-    while (ret < 0 && errno == EINTR);
+        do
+        {
+            ret = fcntl(sock, F_SETFD, opts);
+        }
+        while (ret < 0 && errno == EINTR);
 
-    if (ret < 0)
-    {
-        swSysError("fcntl(%d, SETFD, opts) failed.", sock);
+        if (ret < 0)
+        {
+            swSysError("fcntl(%d, SETFD, opts) failed.", sock);
+        }
     }
 #endif
 }
@@ -1047,6 +1071,52 @@ int swoole_gethostbyname(int flags, char *name, char *addr)
 }
 #endif
 
+int swoole_getaddrinfo(swRequest_getaddrinfo *req)
+{
+    struct addrinfo *result = NULL;
+    struct addrinfo *ptr = NULL;
+    struct addrinfo hints;
+
+    bzero(&hints, sizeof(hints));
+    hints.ai_family = req->family;
+    hints.ai_socktype = req->socktype;
+    hints.ai_protocol = req->protocol;
+
+    int ret = getaddrinfo(req->hostname, req->service, &hints, &result);
+    if (ret != 0)
+    {
+        req->error = ret;
+        return SW_ERR;
+    }
+
+    void *buffer = req->result;
+    int i = 0;
+    for (ptr = result; ptr != NULL; ptr = ptr->ai_next)
+    {
+        switch (ptr->ai_family)
+        {
+        case AF_INET:
+            memcpy(buffer + (i * sizeof(struct sockaddr_in)), ptr->ai_addr, sizeof(struct sockaddr_in));
+            break;
+        case AF_INET6:
+            memcpy(buffer + (i * sizeof(struct sockaddr_in6)), ptr->ai_addr, sizeof(struct sockaddr_in6));
+            break;
+        default:
+            swWarn("unknown socket family[%d].", ptr->ai_family);
+            break;
+        }
+        i++;
+        if (i == SW_DNS_HOST_BUFFER_SIZE)
+        {
+            break;
+        }
+    }
+    freeaddrinfo(result);
+    req->error = 0;
+    req->count = i;
+    return SW_OK;
+}
+
 int swoole_add_function(const char *name, void* func)
 {
     if (SwooleG.functions == NULL)
@@ -1107,6 +1177,26 @@ int swoole_shell_exec(char *command, pid_t *pid)
     return fds[SW_PIPE_READ];
 }
 
+char* swoole_string_format(size_t n, const char *format, ...)
+{
+    char *buf = sw_malloc(n);
+    if (buf == NULL)
+    {
+        return NULL;
+    }
+
+    va_list _va_list;
+    va_start(_va_list, format);
+
+    if (vsnprintf(buf, n, format, _va_list) < 0)
+    {
+        sw_free(buf);
+        return NULL;
+    }
+
+    return buf;
+}
+
 #ifdef HAVE_EXECINFO
 void swoole_print_trace(void)
 {
