@@ -111,6 +111,11 @@ void* swLinkedList_pop(swLinkedList *ll)
 
 void swLinkedList_remove_node(swLinkedList *ll, swLinkedList_node *remove_node)
 {
+    if (ll->num == 0)
+    {
+        return;
+    }
+
     swLinkedList_node *prev = remove_node->prev;
     swLinkedList_node *next = remove_node->next;
 
@@ -176,7 +181,7 @@ void swLinkedList_free(swLinkedList *ll)
     swLinkedList_node *node = ll->head;
     swLinkedList_node *tmp;
 
-    do
+    while (node)
     {
         tmp = node->next;
         if (ll->dtor)
@@ -185,7 +190,7 @@ void swLinkedList_free(swLinkedList *ll)
         }
         sw_free(node);
         node = tmp;
-    } while (node);
+    } 
 
     sw_free(ll);
 }
