@@ -271,6 +271,7 @@ void swProcessPool_shutdown(swProcessPool *pool)
     swWorker *worker;
     SwooleG.running = 0;
 
+	swSignal_none();
     //concurrent kill
     for (i = 0; i < pool->run_worker_num; i++)
     {
@@ -359,13 +360,10 @@ static int swProcessPool_worker_loop(swProcessPool *pool, swWorker *worker)
     else
     {
         task_n = pool->max_request;
-        if (pool->max_request > 10)
+        n = swoole_system_random(1, pool->max_request / 2);
+        if (n > 0)
         {
-            n = swoole_system_random(1, pool->max_request / 2);
-            if (n > 0)
-            {
-                task_n += n;
-            }
+            task_n += n;
         }
     }
 
@@ -489,14 +487,14 @@ int swProcessPool_wait(swProcessPool *pool)
 {
     int pid, new_pid;
     int reload_worker_i = 0;
-    pid_t reload_worker_pid = 0;
     int ret;
     int status;
 
-    swWorker *reload_workers = sw_calloc(pool->worker_num, sizeof(swWorker));
+    swWorker *reload_workers;
+    reload_workers = sw_calloc(pool->worker_num, sizeof(swWorker));
     if (reload_workers == NULL)
     {
-        swError("malloc[reload_workers] failed");
+        swError("[manager] malloc[reload_workers] fail.\n");
         return SW_ERR;
     }
 
@@ -505,30 +503,23 @@ int swProcessPool_wait(swProcessPool *pool)
         pid = wait(&status);
         if (pid < 0)
         {
-            if (SwooleG.running == 0)
+            if (pool->reloading == 0)
+            {
+                swTrace("[Manager] wait failed. Error: %s [%d]", strerror(errno), errno);
+            }
+            else if (pool->reload_flag == 0)
+            {
+                swTrace("[Manager] reload workers.");
+                memcpy(reload_workers, pool->workers, sizeof(swWorker) * pool->worker_num);
+                pool->reload_flag = 1;
+                goto reload_worker;
+            }
+            else if (SwooleG.running == 0)
             {
                 break;
             }
-            if (pool->reloading == 0)
-            {
-                if (errno != EINTR)
-                {
-                    swWarn("[Manager] wait failed. Error: %s [%d]", strerror(errno), errno);
-                }
-                continue;
-            }
-
-            swNotice("reload workers.");
-
-            if (pool->reload_init == 0)
-            {
-                pool->reload_init = 1;
-                memcpy(reload_workers, pool->workers, sizeof(swWorker) * pool->worker_num);
-            }
-
-            goto kill_worker;
         }
-
+        swTrace("[Manager] worker stop.pid=%d", pid);
         if (SwooleG.running == 1)
         {
             swWorker *exit_worker = swHashMap_find_int(pool->map, pid);
@@ -556,32 +547,25 @@ int swProcessPool_wait(swProcessPool *pool)
                 return SW_ERR;
             }
             swHashMap_del_int(pool->map, pid);
-            if (pid == reload_worker_pid)
-            {
-                reload_worker_i++;
-            }
         }
         //reload worker
-        kill_worker: if (pool->reloading == 1)
+        reload_worker:
+        if (pool->reloading == 1)
         {
             //reload finish
             if (reload_worker_i >= pool->worker_num)
             {
-                pool->reloading = pool->reload_init = reload_worker_pid = reload_worker_i = 0;
+                pool->reloading = 0;
+                reload_worker_i = 0;
                 continue;
             }
-            reload_worker_pid = reload_workers[reload_worker_i].pid;
-            ret = kill(reload_worker_pid, SIGTERM);
+            ret = kill(reload_workers[reload_worker_i].pid, SIGTERM);
             if (ret < 0)
             {
-                if (errno == ECHILD)
-                {
-                    reload_worker_i++;
-                    goto kill_worker;
-                }
                 swSysError("[Manager]kill(%d) failed.", reload_workers[reload_worker_i].pid);
                 continue;
             }
+            reload_worker_i++;
         }
     }
     sw_free(reload_workers);
