@@ -200,9 +200,9 @@ int swAioBase_init(int max_aio_events)
 static int swAioBase_thread_onTask(swThreadPool *pool, void *task, int task_len)
 {
     swAio_event *event = task;
-    struct in_addr addr;
+    struct in_addr addr_v4;
+    struct in6_addr addr_v6;
 
-    char *ip_addr;
     int ret = -1;
 
     start_switch:
@@ -246,35 +246,46 @@ static int swAioBase_thread_onTask(swThreadPool *pool, void *task, int task_len)
         }
         break;
     case SW_AIO_DNS_LOOKUP:
+
 #ifndef HAVE_GETHOSTBYNAME2_R
         SwooleAIO.lock.lock(&SwooleAIO.lock);
 #endif
-        ret = swoole_gethostbyname(event->flags == AF_INET6 ? AF_INET6 : AF_INET, event->buf, (char *) &addr);
+        if (event->flags == AF_INET6)
+        {
+            ret = swoole_gethostbyname(AF_INET6, event->buf, (char *) &addr_v6);
+        }
+        else
+        {
+            ret = swoole_gethostbyname(AF_INET, event->buf, (char *) &addr_v4);
+        }
+        bzero(event->buf, event->nbytes);
 #ifndef HAVE_GETHOSTBYNAME2_R
         SwooleAIO.lock.unlock(&SwooleAIO.lock);
 #endif
         if (ret < 0)
         {
             event->error = h_errno;
-            switch (h_errno)
-            {
-            case HOST_NOT_FOUND:
-                bzero(event->buf, event->nbytes);
-                ret = 0;
-                break;
-            default:
-                ret = -1;
-                break;
-            }
         }
         else
         {
-            ip_addr = inet_ntoa(addr);
-            bzero(event->buf, event->nbytes);
-            memcpy(event->buf, ip_addr, strnlen(ip_addr, SW_IP_MAX_LENGTH) + 1);
-            ret = 0;
+            if (inet_ntop(event->flags == AF_INET6 ? AF_INET6 : AF_INET, event->flags == AF_INET6 ? (void *) &addr_v6 : (void *) &addr_v4, event->buf,
+                    event->nbytes) == NULL)
+            {
+                ret = -1;
+                event->error = SW_ERROR_BAD_IPV6_ADDRESS;
+            }
+            else
+            {
+                event->error = 0;
+                ret = 0;
+            }
         }
         break;
+
+    case SW_AIO_GETADDRINFO:
+        event->error = swoole_getaddrinfo((swRequest_getaddrinfo *) event->req);
+        break;
+
     default:
         swWarn("unknow aio task.");
         break;
@@ -285,15 +296,16 @@ static int swAioBase_thread_onTask(swThreadPool *pool, void *task, int task_len)
     {
         if (errno == EINTR || errno == EAGAIN)
         {
+            errno = 0;
             goto start_switch;
         }
-        else
+        else if (event->error == 0)
         {
             event->error = errno;
         }
     }
 
-    swTrace("aio_thread ok. ret=%d", ret);
+    swTrace("aio_thread ok. ret=%d, error=%d", ret, event->error);
     do
     {
         SwooleAIO.lock.lock(&SwooleAIO.lock);
