@@ -143,7 +143,7 @@ static void swReactor_onTimeout_and_Finish(swReactor *reactor)
     //server master
     if (SwooleG.serv && SwooleTG.update_time)
     {
-        swoole_update_time();
+        swServer_master_onTimer(SwooleG.serv);
         int32_t timeout_msec = SwooleG.main_reactor->timeout_msec;
         if (timeout_msec < 0 || timeout_msec > 1000)
         {
@@ -170,6 +170,10 @@ static void swReactor_onTimeout_and_Finish(swReactor *reactor)
         {
             reactor->running = 0;
         }
+        if (reactor->running == 0 && reactor->can_exit)
+        {
+            reactor->running = reactor->can_exit(reactor);
+        }
     }
 
 #ifdef SW_USE_MALLOC_TRIM
@@ -251,6 +255,12 @@ int swReactor_write(swReactor *reactor, int fd, void *buf, int n)
         socket->buffer_size = SwooleG.socket_buffer_size;
     }
 
+    if (socket->nonblock == 0)
+    {
+        swoole_fcntl_set_option(fd, 1, -1);
+        socket->nonblock = 1;
+    }
+
     if (n > socket->buffer_size)
     {
         swoole_error_log(SW_LOG_WARNING, SW_ERROR_PACKAGE_LENGTH_TOO_LARGE, "data is too large, cannot exceed buffer size.");
