@@ -1039,11 +1039,6 @@ PHP_FUNCTION(swoole_async_set)
         convert_to_boolean(v);
         SWOOLE_G(display_errors) = 0;
     }
-    if (php_swoole_array_get_value(vht, "aio_chunk_size", v))
-    {
-        convert_to_string(v);
-        SwooleG.dns_server_v4 = sw_strndup(Z_STRVAL_P(v), Z_STRLEN_P(v));
-    }
     if (php_swoole_array_get_value(vht, "socket_dontwait", v))
     {
         convert_to_boolean(v);
@@ -1064,15 +1059,6 @@ PHP_FUNCTION(swoole_async_set)
         convert_to_boolean(v);
         SwooleG.use_async_resolver = Z_BVAL_P(v);
     }
-    if (php_swoole_array_get_value(vht, "max_coroutine", v))
-    {
-        convert_to_long(v);
-        COROG.max_coro_num = (int) Z_LVAL_P(v);
-        if (COROG.max_coro_num <= 0)
-        {
-            COROG.max_coro_num = DEFAULT_MAX_CORO_NUM;
-        }
-    }
 #if defined(HAVE_REUSEPORT) && defined(HAVE_EPOLL)
     //reuse port
     if (php_swoole_array_get_value(vht, "enable_reuse_port", v))
