@@ -214,6 +214,7 @@ void php_zlib_free(voidpf opaque, voidpf address);
 static PHP_METHOD(swoole_http_server, on);
 static PHP_METHOD(swoole_http_server, start);
 
+static PHP_METHOD(swoole_http_request, getData);
 static PHP_METHOD(swoole_http_request, rawcontent);
 static PHP_METHOD(swoole_http_request, __destruct);
 
@@ -385,6 +386,7 @@ const zend_function_entry swoole_http_server_methods[] =
 const zend_function_entry swoole_http_request_methods[] =
 {
     PHP_ME(swoole_http_request, rawcontent, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http_request, getData, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_FALIAS(__sleep, swoole_unsupport_serialize, NULL)
     PHP_FALIAS(__wakeup, swoole_unsupport_serialize, NULL)
     PHP_ME(swoole_http_request, __destruct, arginfo_swoole_http_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
@@ -621,21 +623,28 @@ static int http_request_on_header_value(php_http_parser *parser, const char *at,
             {
                 offset = sizeof("multipart/form-data;") - 1;
 
-                while (at[offset] == ' ') {
+                while (at[offset] == ' ')
+                {
                     offset += 1;
                 }
 
                 offset += sizeof("boundary=") - 1;
 
                 int boundary_len = length - offset;
+                char *boundary_str = (char *) at + length - boundary_len;
 
                 if (boundary_len <= 0)
                 {
                     swWarn("invalid multipart/form-data body.", ctx->fd);
                     return 0;
                 }
+                if (boundary_len >= 2 && boundary_str[0] == '"' && *(boundary_str + boundary_len - 1) == '"')
+                {
+                    boundary_str++;
+                    boundary_len -= 2;
+                }
 
-                swoole_http_parse_form_data(ctx, at + length - boundary_len, boundary_len TSRMLS_CC);
+                swoole_http_parse_form_data(ctx, boundary_str, boundary_len TSRMLS_CC);
             }
         }
     }
@@ -941,7 +950,12 @@ static int http_request_on_body(php_http_parser *parser, const char *at, size_t
     else if (ctx->mt_parser != NULL)
     {
         multipart_parser *multipart_parser = ctx->mt_parser;
-        size_t n = multipart_parser_execute(multipart_parser, at, length);
+        char *c = (char *) at;
+        while (*c == '\r' && *(c + 1) == '\n')
+        {
+            c += 2;
+        }
+        size_t n = multipart_parser_execute(multipart_parser, c, length);
         if (n != length)
         {
             swoole_php_fatal_error(E_WARNING, "parse multipart body failed.");
@@ -1243,6 +1257,13 @@ void swoole_http_server_init(int module_number TSRMLS_DC)
     swoole_http_request_class_entry_ptr = zend_register_internal_class(&swoole_http_request_ce TSRMLS_CC);
     SWOOLE_CLASS_ALIAS(swoole_http_request, "Swoole\\Http\\Request");
 
+    if (SWOOLE_G(use_shortname))
+    {
+        zend_register_class_alias("Co\\Http\\Server", swoole_http_server_class_entry_ptr);
+        zend_register_class_alias("Co\\Http\\Request", swoole_http_request_class_entry_ptr);
+        zend_register_class_alias("Co\\Http\\Response", swoole_http_response_class_entry_ptr);
+    }
+
     zend_declare_property_long(swoole_http_request_class_entry_ptr, SW_STRL("fd")-1, 0,  ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_http_request_class_entry_ptr, SW_STRL("header")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
     zend_declare_property_null(swoole_http_request_class_entry_ptr, SW_STRL("server")-1, ZEND_ACC_PUBLIC TSRMLS_CC);
@@ -1586,6 +1607,9 @@ static PHP_METHOD(swoole_http_server, start)
     {
         SW_ALLOC_INIT_ZVAL(zsetting);
         array_init(zsetting);
+#ifdef HT_ALLOW_COW_VIOLATION
+        HT_ALLOW_COW_VIOLATION(Z_ARRVAL_P(zsetting));
+#endif
         zend_update_property(swoole_server_class_entry_ptr, getThis(), ZEND_STRL("setting"), zsetting TSRMLS_CC);
     }
 
@@ -1623,13 +1647,6 @@ static PHP_METHOD(swoole_http_server, start)
 
 static PHP_METHOD(swoole_http_request, rawcontent)
 {
-    zval *zfd = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("fd"), 0 TSRMLS_CC);
-    if (ZVAL_IS_NULL(zfd))
-    {
-        swoole_php_error(E_WARNING, "http client is not existed.");
-        RETURN_FALSE;
-    }
-
     http_context *ctx = http_get_context(getThis(), 0 TSRMLS_CC);
     if (!ctx)
     {
@@ -1654,6 +1671,19 @@ static PHP_METHOD(swoole_http_request, rawcontent)
     }
 }
 
+static PHP_METHOD(swoole_http_request, getData)
+{
+    zval *zdata = swoole_get_property(getThis(), 0);
+    if (zdata)
+    {
+        SW_RETURN_STRINGL(Z_STRVAL_P(zdata), Z_STRLEN_P(zdata), 1);
+    }
+    else
+    {
+        RETURN_FALSE;
+    }
+}
+
 static PHP_METHOD(swoole_http_request, __destruct)
 {
     zval *ztmpfiles = sw_zend_read_property(swoole_http_request_class_entry_ptr, getThis(), ZEND_STRL("tmpfiles"), 1 TSRMLS_CC);
@@ -2610,6 +2640,8 @@ static PHP_METHOD(swoole_http_response, __destruct)
         }
         else
         {
+            context->response.status = 500;
+
             zval *zobject = getThis();
             zval *retval = NULL;
             sw_zend_call_method_with_0_params(&zobject, swoole_http_response_class_entry_ptr, NULL, "end", &retval);
