@@ -334,12 +334,6 @@ static void mysql_client_free(mysql_client *client, zval* zobject)
     client->connected = 0;
 }
 
-void mysql_statement_free(mysql_client *client)
-{
-    efree(client->statement);
-    client->statement = NULL;
-}
-
 static void mysql_columns_free(mysql_client *client)
 {
     int i;
@@ -673,6 +667,7 @@ static int mysql_parse_prepare_result(mysql_client *client, char *buf, size_t n_
     buf += 1;
     stmt->warning_count = mysql_uint2korr(buf);
     client->statement = stmt;
+    stmt->client = client;
 
     swTraceLog(SW_TRACE_MYSQL_CLIENT, "id=%d, field_count=%d, param_count=%d, warning_count=%d.", stmt->id, stmt->field_count, stmt->param_count,
             stmt->warning_count);
@@ -687,13 +682,12 @@ static int mysql_decode_row(mysql_client *client, char *buf, int packet_len)
     ulong_t len;
     char nul;
 
-#ifdef SW_MYSQL_STRICT_TYPE
     mysql_row row;
     char value_buffer[32];
     bzero(&row, sizeof(row));
     char *error;
-    char mem;
-#endif
+    //unused
+    //char mem;
 
     zval *result_array = client->response.result_array;
     zval *row_array = NULL;
@@ -753,6 +747,7 @@ static int mysql_decode_row(mysql_client *client, char *buf, int packet_len)
         case SW_MYSQL_TYPE_TIMESTAMP:
         case SW_MYSQL_TYPE_DATETIME:
         case SW_MYSQL_TYPE_DATE:
+        case SW_MYSQL_TYPE_JSON:
             sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
             break;
         /* Integer */
@@ -760,62 +755,75 @@ static int mysql_decode_row(mysql_client *client, char *buf, int packet_len)
         case SW_MYSQL_TYPE_SHORT:
         case SW_MYSQL_TYPE_INT24:
         case SW_MYSQL_TYPE_LONG:
-#ifdef SW_MYSQL_STRICT_TYPE
-            memcpy(value_buffer, buf + read_n, len);
-            value_buffer[len] = 0;
-            row.sint = strtol(value_buffer, &error, 10);
-            if (*error != '\0')
+            if(client->connector.strict_type)
             {
-                return -SW_MYSQL_ERR_CONVLONG;
+                memcpy(value_buffer, buf + read_n, len);
+                value_buffer[len] = 0;
+                row.sint = strtol(value_buffer, &error, 10);
+                if (*error != '\0')
+                {
+                    return -SW_MYSQL_ERR_CONVLONG;
+                }
+                add_assoc_long(row_array, client->response.columns[i].name, row.sint);
+            }
+            else
+            {
+                sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
+
             }
-            add_assoc_long(row_array, client->response.columns[i].name, row.sint);
-#else
-            sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
-#endif
             break;
         case SW_MYSQL_TYPE_LONGLONG:
-#ifdef SW_MYSQL_STRICT_TYPE
-            memcpy(value_buffer, buf + read_n, len);
-            value_buffer[len] = 0;
-            row.sbigint = strtoll(value_buffer, &error, 10);
-            if (*error != '\0')
+            if(client->connector.strict_type) {
+                memcpy(value_buffer, buf + read_n, len);
+                value_buffer[len] = 0;
+                row.sbigint = strtoll(value_buffer, &error, 10);
+                if (*error != '\0') {
+                    return -SW_MYSQL_ERR_CONVLONG;
+                }
+                add_assoc_long(row_array, client->response.columns[i].name, row.sbigint);
+            }
+            else
             {
-                return -SW_MYSQL_ERR_CONVLONG;
+                sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
+
             }
-            add_assoc_long(row_array, client->response.columns[i].name, row.sbigint);
-#else
-            sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
-#endif
             break;
         case SW_MYSQL_TYPE_FLOAT:
-#ifdef SW_MYSQL_STRICT_TYPE
-            memcpy(value_buffer, buf + read_n, len);
-            value_buffer[len] = 0;
-            row.mfloat = strtof(value_buffer, &error);
-            if (*error != '\0')
+            if(client->connector.strict_type) {
+                memcpy(value_buffer, buf + read_n, len);
+                value_buffer[len] = 0;
+                row.mfloat = strtof(value_buffer, &error);
+                if (*error != '\0') {
+                    return -SW_MYSQL_ERR_CONVFLOAT;
+                }
+                add_assoc_double(row_array, client->response.columns[i].name, row.mfloat);
+            }
+            else
             {
-                return -SW_MYSQL_ERR_CONVFLOAT;
+                sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
             }
-            add_assoc_double(row_array, client->response.columns[i].name, row.mfloat);
-#else
-            sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
-#endif
             break;
 
         case SW_MYSQL_TYPE_DOUBLE:
-#ifdef SW_MYSQL_STRICT_TYPE
-            memcpy(value_buffer, buf + read_n, len);
-            value_buffer[len] = 0;
-            row.mdouble = strtod(value_buffer, &error);
-            if (*error != '\0')
+            if(client->connector.strict_type) {
+                memcpy(value_buffer, buf + read_n, len);
+                value_buffer[len] = 0;
+                row.mdouble = strtod(value_buffer, &error);
+                if (*error != '\0') {
+                    return -SW_MYSQL_ERR_CONVDOUBLE;
+                }
+                add_assoc_double(row_array, client->response.columns[i].name, row.mdouble);
+            }
+            else
             {
-                return -SW_MYSQL_ERR_CONVDOUBLE;
+                sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
+
             }
-            add_assoc_double(row_array, client->response.columns[i].name, row.mdouble);
-#else
-            sw_add_assoc_stringl(row_array, client->response.columns[i].name, buf + read_n, len, 1);
-#endif
             break;
+
+        default:
+            swWarn("unknown field type[%d].", type);
+            return -1;
         }
         read_n += len;
     }
@@ -856,15 +864,21 @@ static int mysql_decode_datetime(char *buf, char *result)
     return n;
 }
 
-static void mysql_decode_time(char *buf, char *result)
+static int mysql_decode_time(char *buf, char *result)
 {
-    uint8_t h, m, s;
+    uint8_t h = 0, m = 0, s = 0;
 
-    h = *(uint8_t *) (buf + 6);
-    m = *(uint8_t *) (buf + 7);
-    s = *(uint8_t *) (buf + 8);
+    uint8_t n = *(uint8_t *) (buf);
+    if (n != 0)
+    {
+        h = *(uint8_t *) (buf + 6);
+        m = *(uint8_t *) (buf + 7);
+        s = *(uint8_t *) (buf + 8);
+    }
 
     snprintf(result, DATETIME_MAX_SIZE, "%02d:%02d:%02d", h, m, s);
+
+    return n;
 }
 
 static int mysql_decode_date(char *buf, char *result)
@@ -915,15 +929,21 @@ static int mysql_decode_row_prepare(mysql_client *client, char *buf, int packet_
 
     for (i = 0; i < client->response.num_column; i++)
     {
+        /* to check Null-Bitmap @see https://dev.mysql.com/doc/internals/en/null-bitmap.html */
+        if( ( (buf - null_count + 1)[((i+2)/8)] & (0x01 << ((i+2)%8)) ) != 0 ){
+            swTraceLog(SW_TRACE_MYSQL_CLIENT, "value: %s is null ,flag2", client->response.columns[i].name);
+            add_assoc_null(row_array, client->response.columns[i].name);
+            continue;
+        }
+
         int type = client->response.columns[i].type;
         swTraceLog(SW_TRACE_MYSQL_CLIENT, "value: name=%s, type=%d", client->response.columns[i].name, type);
         switch (type)
         {
         /* Date Time */
         case SW_MYSQL_TYPE_TIME:
-            mysql_decode_time(buf + read_n, datetime_buffer);
+            len = mysql_decode_time(buf + read_n, datetime_buffer);
             sw_add_assoc_stringl(row_array, client->response.columns[i].name, datetime_buffer, 8, 1);
-            len = 8;
             swTraceLog(SW_TRACE_MYSQL_CLIENT, "%s=%s", client->response.columns[i].name, datetime_buffer);
             break;
 
@@ -959,6 +979,7 @@ static int mysql_decode_row_prepare(mysql_client *client, char *buf, int packet_
         case SW_MYSQL_TYPE_DECIMAL:
         case SW_MYSQL_TYPE_NEWDECIMAL:
         case SW_MYSQL_TYPE_BIT:
+        case SW_MYSQL_TYPE_JSON:
         case SW_MYSQL_TYPE_STRING:
         case SW_MYSQL_TYPE_VAR_STRING:
         case SW_MYSQL_TYPE_VARCHAR:
@@ -1016,6 +1037,10 @@ static int mysql_decode_row_prepare(mysql_client *client, char *buf, int packet_
             len = sizeof(row.mdouble);
             swTraceLog(SW_TRACE_MYSQL_CLIENT, "%s=%f", client->response.columns[i].name, row.mdouble);
             break;
+
+        default:
+            swWarn("unknown field type[%d].", type);
+            return -1;
         }
         read_n += len;
     }
@@ -1148,7 +1173,7 @@ static sw_inline int mysql_read_rows(mysql_client *client)
             return SW_ERR;
         }
 
-        if (client->prepare_state == SW_MYSQL_PREPARE_READY)
+        if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
         {
             ret = mysql_decode_row_prepare(client, buffer, client->response.packet_length);
         }
@@ -1566,7 +1591,6 @@ int mysql_response(mysql_client *client)
             {
                 if (client->cmd == SW_MYSQL_COM_STMT_PREPARE)
                 {
-                    client->prepare_state = SW_MYSQL_PREPARE_READY;
                     mysql_columns_free(client);
                     return SW_OK;
                 }
@@ -1597,7 +1621,6 @@ int mysql_response(mysql_client *client)
             }
             else
             {
-                client->prepare_state = SW_MYSQL_PREPARE_READY;
                 mysql_columns_free(client);
                 return SW_OK;
             }
@@ -1818,6 +1841,22 @@ static PHP_METHOD(swoole_mysql, connect)
         connector->character_set = 0;
     }
 
+    if (php_swoole_array_get_value(_ht, "strict_type", value))
+    {
+#if PHP_MAJOR_VERSION < 7
+        if(Z_TYPE_P(value) == IS_BOOL && Z_BVAL_P(value) == 1)
+#else
+        if(Z_TYPE_P(value) == IS_TRUE)
+#endif
+        {
+            connector->strict_type = 1;
+        }else{
+            connector->strict_type = 0;
+        }
+    } else{
+        connector->strict_type = 0;
+    }
+
     swClient *cli = emalloc(sizeof(swClient));
     int type = SW_SOCK_TCP;
 
@@ -2562,6 +2601,11 @@ static PHP_METHOD(swoole_mysql, escape)
     }
 
     const MYSQLND_CHARSET* cset = mysqlnd_find_charset_nr(client->connector.character_set);
+    if (cset == NULL)
+    {
+        swoole_php_fatal_error(E_ERROR, "unknown mysql charset[%s].", client->connector.character_set);
+        RETURN_FALSE;
+    }
     int newstr_len = mysqlnd_cset_escape_slashes(cset, newstr, str.str, str.length TSRMLS_CC);
     if (newstr_len < 0)
     {
