@@ -75,13 +75,6 @@ enum mysql_read_state
     SW_MYSQL_STATE_CLOSED,
 };
 
-enum mysql_prepare_state
-{
-    SW_MYSQL_PREPARE_NONE,
-    SW_MYSQL_PREPARE_BEGIN,
-    SW_MYSQL_PREPARE_READY,
-};
-
 enum mysql_error_code
 {
     SW_MYSQL_ERR_PROTOCOL_ERROR = 1,
@@ -130,6 +123,7 @@ enum mysql_field_types
     SW_MYSQL_TYPE_NEWDATE,
     SW_MYSQL_TYPE_VARCHAR,
     SW_MYSQL_TYPE_BIT,
+    SW_MYSQL_TYPE_JSON = 245,
     SW_MYSQL_TYPE_NEWDECIMAL = 246,
     SW_MYSQL_TYPE_ENUM = 247,
     SW_MYSQL_TYPE_SET = 248,
@@ -182,6 +176,7 @@ typedef struct
     char *user;
     char *password;
     char *database;
+    zend_bool strict_type;
 
     zend_size_t host_len;
     zend_size_t user_len;
@@ -251,7 +246,8 @@ typedef struct
     uint16_t param_count;
     uint16_t warning_count;
     uint16_t unreaded_param_count;
-
+    struct _mysql_client *client;
+    zval *object;
 } mysql_statement;
 
 typedef struct
@@ -275,7 +271,7 @@ typedef struct
     zval *result_array;
 } mysql_response_t;
 
-typedef struct
+typedef struct _mysql_client
 {
 #ifdef SW_COROUTINE
     zend_bool defer;
@@ -285,7 +281,6 @@ typedef struct
     int cid;
 #endif
     uint8_t state;
-    uint8_t prepare_state;
     uint8_t handshake;
     uint8_t cmd;
     swString *buffer;
@@ -299,13 +294,14 @@ typedef struct
     uint32_t strict;
 
     mysql_connector connector;
+    mysql_statement *statement;
+    swLinkedList *statement_list;
 
 #if PHP_MAJOR_VERSION >= 7
     zval _object;
     zval _onClose;
 #endif
     mysql_response_t response;
-    mysql_statement *statement;
 } mysql_client;
 
 #define mysql_uint2korr(A)  (uint16_t) (((uint16_t) ((zend_uchar) (A)[0])) +\
@@ -461,6 +457,5 @@ static sw_inline int mysql_length_coded_binary(char *m, ulong_t *r, char *nul, i
 }
 
 int mysql_query(zval *zobject, mysql_client *client, swString *sql, zval *callback TSRMLS_DC);
-void mysql_statement_free(mysql_client *client);
 
 #endif /* SWOOLE_MYSQL_H_ */
