@@ -39,11 +39,12 @@ static PHP_METHOD(swoole_mysql_coro, begin);
 static PHP_METHOD(swoole_mysql_coro, commit);
 static PHP_METHOD(swoole_mysql_coro, rollback);
 static PHP_METHOD(swoole_mysql_coro, prepare);
-static PHP_METHOD(swoole_mysql_coro, execute);
 static PHP_METHOD(swoole_mysql_coro, setDefer);
 static PHP_METHOD(swoole_mysql_coro, getDefer);
 static PHP_METHOD(swoole_mysql_coro, close);
 
+static PHP_METHOD(swoole_mysql_coro_statement, __destruct);
+static PHP_METHOD(swoole_mysql_coro_statement, execute);
 
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_void, 0, 0, 0)
 ZEND_END_ARG_INFO()
@@ -70,10 +71,6 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_prepare, 0, 0, 2)
     ZEND_ARG_INFO(0, query)
 ZEND_END_ARG_INFO()
 
-ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_execute, 0, 0, 2)
-    ZEND_ARG_INFO(0, query)
-ZEND_END_ARG_INFO()
-
 ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_setDefer, 0, 0, 0)
     ZEND_ARG_INFO(0, defer)
 ZEND_END_ARG_INFO()
@@ -85,12 +82,19 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_escape, 0, 0, 1)
 ZEND_END_ARG_INFO()
 #endif
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_mysql_coro_statement_execute, 0, 0, 1)
+    ZEND_ARG_INFO(0, params)
+ZEND_END_ARG_INFO()
+
 static zend_class_entry swoole_mysql_coro_ce;
 static zend_class_entry *swoole_mysql_coro_class_entry_ptr;
 
 static zend_class_entry swoole_mysql_coro_exception_ce;
 static zend_class_entry *swoole_mysql_coro_exception_class_entry_ptr;
 
+static zend_class_entry swoole_mysql_coro_statement_ce;
+static zend_class_entry *swoole_mysql_coro_statement_class_entry_ptr;
+
 static const zend_function_entry swoole_mysql_coro_methods[] =
 {
     PHP_ME(swoole_mysql_coro, __construct, arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_CTOR)
@@ -105,7 +109,6 @@ static const zend_function_entry swoole_mysql_coro_methods[] =
     PHP_ME(swoole_mysql_coro, commit, arginfo_swoole_mysql_coro_commit, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, rollback, arginfo_swoole_mysql_coro_rollback, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, prepare, arginfo_swoole_mysql_coro_prepare, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_mysql_coro, execute, arginfo_swoole_mysql_coro_execute, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, setDefer, arginfo_swoole_mysql_coro_setDefer, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, getDefer, arginfo_swoole_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_mysql_coro, close, arginfo_swoole_void, ZEND_ACC_PUBLIC)
@@ -114,6 +117,15 @@ static const zend_function_entry swoole_mysql_coro_methods[] =
     PHP_FE_END
 };
 
+static const zend_function_entry swoole_mysql_coro_statement_methods[] =
+{
+    PHP_ME(swoole_mysql_coro_statement, execute, arginfo_swoole_mysql_coro_statement_execute, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_mysql_coro_statement, __destruct, arginfo_swoole_void, ZEND_ACC_PUBLIC | ZEND_ACC_DTOR)
+    PHP_FALIAS(__sleep, swoole_unsupport_serialize, NULL)
+    PHP_FALIAS(__wakeup, swoole_unsupport_serialize, NULL)
+    PHP_FE_END
+};
+
 static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event);
 static int swoole_mysql_coro_onWrite(swReactor *reactor, swEvent *event);
 static int swoole_mysql_coro_onError(swReactor *reactor, swEvent *event);
@@ -124,11 +136,24 @@ extern swString *mysql_request_buffer;
 
 void swoole_mysql_coro_init(int module_number TSRMLS_DC)
 {
-    SWOOLE_INIT_CLASS_ENTRY(swoole_mysql_coro_ce, "swoole_mysql_coro", "Swoole\\Coroutine\\MySQL", swoole_mysql_coro_methods);
+    INIT_CLASS_ENTRY(swoole_mysql_coro_ce, "Swoole\\Coroutine\\MySQL", swoole_mysql_coro_methods);
     swoole_mysql_coro_class_entry_ptr = zend_register_internal_class(&swoole_mysql_coro_ce TSRMLS_CC);
 
-    SWOOLE_INIT_CLASS_ENTRY(swoole_mysql_coro_exception_ce, "swoole_mysql_coro_exception", "Swoole\\Coroutine\\MySQL\\Exception", NULL);
-    swoole_mysql_coro_exception_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_mysql_coro_exception_ce, zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);
+    INIT_CLASS_ENTRY(swoole_mysql_coro_statement_ce, "Swoole\\Coroutine\\MySQL\\Statement",
+            swoole_mysql_coro_statement_methods);
+    swoole_mysql_coro_statement_class_entry_ptr = zend_register_internal_class(
+            &swoole_mysql_coro_statement_ce TSRMLS_CC);
+
+    INIT_CLASS_ENTRY(swoole_mysql_coro_exception_ce, "Swoole\\Coroutine\\MySQL\\Exception", NULL);
+    swoole_mysql_coro_exception_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_mysql_coro_exception_ce,
+            zend_exception_get_default(TSRMLS_C), NULL TSRMLS_CC);
+
+    if (SWOOLE_G(use_shortname))
+    {
+        zend_register_class_alias("Co\\MySQL", swoole_mysql_coro_class_entry_ptr);
+        zend_register_class_alias("Co\\MySQL\\Statement", swoole_mysql_coro_statement_class_entry_ptr);
+        zend_register_class_alias("Co\\MySQL\\Exception", swoole_mysql_coro_exception_class_entry_ptr);
+    }
 
     zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, SW_STRL("serverInfo") - 1, "", ZEND_ACC_PRIVATE TSRMLS_CC);
 	zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, SW_STRL("sock") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
@@ -139,6 +164,11 @@ void swoole_mysql_coro_init(int module_number TSRMLS_DC)
 	zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, SW_STRL("insert_id") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
 	zend_declare_property_string(swoole_mysql_coro_class_entry_ptr, SW_STRL("error") - 1, "", ZEND_ACC_PUBLIC TSRMLS_CC);
 	zend_declare_property_long(swoole_mysql_coro_class_entry_ptr, SW_STRL("errno") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+
+    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, SW_STRL("affected_rows") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, SW_STRL("insert_id") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
+    zend_declare_property_string(swoole_mysql_coro_statement_class_entry_ptr, SW_STRL("error") - 1, "", ZEND_ACC_PUBLIC TSRMLS_CC);
+    zend_declare_property_long(swoole_mysql_coro_statement_class_entry_ptr, SW_STRL("errno") - 1, 0, ZEND_ACC_PUBLIC TSRMLS_CC);
 }
 
 int mysql_query(zval *zobject, mysql_client *client, swString *sql, zval *callback TSRMLS_DC);
@@ -158,11 +188,6 @@ static int swoole_mysql_coro_close(zval *this)
         return FAILURE;
     }
 
-    if (client->statement)
-    {
-        mysql_statement_free(client);
-    }
-
     zend_update_property_bool(swoole_mysql_coro_class_entry_ptr, this, ZEND_STRL("connected"), 0 TSRMLS_CC);
     SwooleG.main_reactor->del(SwooleG.main_reactor, client->fd);
 
@@ -176,6 +201,23 @@ static int swoole_mysql_coro_close(zval *this)
         client->cli->timeout_id = 0;
     }
 
+    if (client->statement_list)
+    {
+        swLinkedList_node *node = client->statement_list->head;
+        while(node)
+        {
+            mysql_statement *stmt = node->data;
+            if (stmt->object)
+            {
+                swoole_set_object(stmt->object, NULL);
+                efree(stmt->object);
+            }
+            efree(stmt);
+            node = node->next;
+        }
+        swLinkedList_free(client->statement_list);
+    }
+
     client->cli->close(client->cli);
     swClient_free(client->cli);
     efree(client->cli);
@@ -200,28 +242,24 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
         return SW_ERR;
     }
 
-    if (client->prepare_state != SW_MYSQL_PREPARE_READY)
+    mysql_statement *statement = swoole_get_object(zobject);
+    if (!statement)
     {
         swoole_php_fatal_error(E_WARNING, "mysql preparation is not ready.");
         return SW_ERR;
     }
 
-    if (!client->statement)
+    if (php_swoole_array_length(params) != statement->param_count)
     {
-        swoole_php_fatal_error(E_WARNING, "mysql preparation is not ready.");
-        return SW_ERR;
-    }
-
-    if (php_swoole_array_length(params) != client->statement->param_count)
-    {
-        swoole_php_fatal_error(E_WARNING, "expects %d parameter, %d given.", client->statement->param_count,
-                php_swoole_array_length(params));
+        swoole_php_fatal_error(E_WARNING, "mysql statement#%d expects %d parameter, %d given.", statement->id,
+                statement->param_count, php_swoole_array_length(params));
         return SW_ERR;
     }
 
     swString_clear(mysql_request_buffer);
 
     client->cmd = SW_MYSQL_COM_STMT_EXECUTE;
+    client->statement = statement;
 
     bzero(mysql_request_buffer->str, 5);
     //command
@@ -231,7 +269,7 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
     p += 5;
 
     // stmt.id
-    mysql_int4store(p, 1);
+    mysql_int4store(p, statement->id);
     p += 4;
     // flags = CURSOR_TYPE_NO_CURSOR
     mysql_int1store(p, 0);
@@ -253,12 +291,11 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
     p += 1;
     mysql_request_buffer->length += 1;
 
+    int i;
+    for (i = 0; i < statement->param_count; i++)
     {
-        zval *value;
-        SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(params), value)
-            mysql_int2store(p, SW_MYSQL_TYPE_VAR_STRING);
-            p += 2;
-        SW_HASHTABLE_FOREACH_END();
+        mysql_int2store(p, SW_MYSQL_TYPE_VAR_STRING);
+        p += 2;
     }
 
     mysql_request_buffer->length += php_swoole_array_length(params) * 2;
@@ -268,17 +305,41 @@ static int swoole_mysql_coro_execute(zval *zobject, mysql_client *client, zval *
 
     {
         zval *value;
+        zval _value;
         SW_HASHTABLE_FOREACH_START(Z_ARRVAL_P(params), value)
+            ZVAL_DUP(&_value, value);
+            value = &_value;
             convert_to_string(value);
+            if (Z_STRLEN_P(value) > 0xffff)
+            {
+                buf[0] = SW_MYSQL_TYPE_VAR_STRING;
+                if (swString_append_ptr(mysql_request_buffer, buf, 1) < 0)
+                {
+                    zval_dtor(value);
+                    return SW_ERR;
+                }
+            }
+            else if (Z_STRLEN_P(value) > 250)
+            {
+                buf[0] = SW_MYSQL_TYPE_BLOB;
+                if (swString_append_ptr(mysql_request_buffer, buf, 1) < 0)
+                {
+                    zval_dtor(value);
+                    return SW_ERR;
+                }
+            }
             lval = mysql_write_lcb(buf, Z_STRLEN_P(value));
             if (swString_append_ptr(mysql_request_buffer, buf, lval) < 0)
             {
+                zval_dtor(value);
                 return SW_ERR;
             }
             if (swString_append_ptr(mysql_request_buffer, Z_STRVAL_P(value), Z_STRLEN_P(value)) < 0)
             {
+                zval_dtor(value);
                 return SW_ERR;
             }
+            zval_dtor(value);
         SW_HASHTABLE_FOREACH_END();
     }
 
@@ -432,6 +493,22 @@ static PHP_METHOD(swoole_mysql_coro, connect)
         connector->character_set = SW_MYSQL_DEFAULT_CHARSET;
     }
 
+    if (php_swoole_array_get_value(_ht, "strict_type", value))
+    {
+#if PHP_MAJOR_VERSION < 7
+        if(Z_TYPE_P(value) == IS_BOOL && Z_BVAL_P(value) == 1)
+#else
+        if(Z_TYPE_P(value) == IS_TRUE)
+#endif
+        {
+            connector->strict_type = 1;
+        }else{
+            connector->strict_type = 0;
+        }
+    } else{
+        connector->strict_type = 0;
+    }
+
     swClient *cli = emalloc(sizeof(swClient));
     int type = SW_SOCK_TCP;
 
@@ -838,21 +915,12 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
         RETURN_FALSE;
     }
 
-    //free the last statement
-    if (client->statement)
-    {
-        mysql_statement_free(client);
-    }
-
-    client->prepare_state = SW_MYSQL_PREPARE_BEGIN;
+    client->cmd = SW_MYSQL_COM_STMT_PREPARE;
 
     swString_clear(mysql_request_buffer);
 
-    client->cmd = SW_MYSQL_COM_STMT_PREPARE;
-
     if (mysql_prepare(&sql, mysql_request_buffer) < 0)
     {
-        client->prepare_state = SW_MYSQL_PREPARE_NONE;
         RETURN_FALSE;
     }
     //send query
@@ -884,7 +952,7 @@ static PHP_METHOD(swoole_mysql_coro, prepare)
     }
 }
 
-static PHP_METHOD(swoole_mysql_coro, execute)
+static PHP_METHOD(swoole_mysql_coro_statement, execute)
 {
     zval *params;
     if (zend_parse_parameters(ZEND_NUM_ARGS()TSRMLS_CC, "a", &params) == FAILURE)
@@ -892,11 +960,13 @@ static PHP_METHOD(swoole_mysql_coro, execute)
         RETURN_FALSE;
     }
 
-    mysql_client *client = swoole_get_object(getThis());
-    if (!client)
+    mysql_statement *stmt = swoole_get_object(getThis());
+    if (!stmt)
     {
-        return;
+        RETURN_FALSE;
     }
+
+    mysql_client *client = stmt->client;
     if (!client->cli)
     {
         swoole_php_fatal_error(E_WARNING, "mysql connection#%d is closed.", client->fd);
@@ -907,7 +977,8 @@ static PHP_METHOD(swoole_mysql_coro, execute)
     {
         RETURN_FALSE;
     }
-    php_context *context = swoole_get_property(getThis(), 0);
+
+    php_context *context = swoole_get_property(client->object, 0);
     double timeout = client->connector.timeout;
     if (timeout > 0)
     {
@@ -927,6 +998,17 @@ static PHP_METHOD(swoole_mysql_coro, execute)
     coro_yield();
 }
 
+static PHP_METHOD(swoole_mysql_coro_statement, __destruct)
+{
+    mysql_statement *stmt = swoole_get_object(getThis());
+    if (!stmt)
+    {
+        return;
+    }
+    efree(stmt->object);
+    stmt->object = NULL;
+}
+
 #ifdef SW_USE_MYSQLND
 static PHP_METHOD(swoole_mysql_coro, escape)
 {
@@ -965,6 +1047,11 @@ static PHP_METHOD(swoole_mysql_coro, escape)
     }
 
     const MYSQLND_CHARSET* cset = mysqlnd_find_charset_nr(client->connector.character_set);
+    if (cset == NULL)
+    {
+        swoole_php_fatal_error(E_ERROR, "unknown mysql charset[%s].", client->connector.character_set);
+        RETURN_FALSE;
+    }
     int newstr_len = mysqlnd_cset_escape_slashes(cset, newstr, str.str, str.length TSRMLS_CC);
     if (newstr_len < 0)
     {
@@ -1091,13 +1178,15 @@ static void swoole_mysql_coro_onConnect(mysql_client *client TSRMLS_DC)
         ZVAL_BOOL(result, 1);
     }
 
-	php_context *sw_current_context = swoole_get_property(zobject, 0);
-	int ret = coro_resume(sw_current_context, result, &retval);
+    client->cid = 0;
+
+    php_context *sw_current_context = swoole_get_property(zobject, 0);
+    int ret = coro_resume(sw_current_context, result, &retval);
     sw_zval_ptr_dtor(&result);
-	if (ret == CORO_END && retval)
-	{
-		sw_zval_ptr_dtor(&retval);
-	}
+    if (ret == CORO_END && retval)
+    {
+        sw_zval_ptr_dtor(&retval);
+    }
 }
 
 static void swoole_mysql_coro_onTimeout(swTimer *timer, swTimer_node *tnode)
@@ -1371,15 +1460,40 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
             //remove from eventloop
             //reactor->del(reactor, event->fd);
 
-            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("affected_rows"), client->response.affected_rows TSRMLS_CC);
-            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("insert_id"), client->response.insert_id TSRMLS_CC);
+            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("affected_rows"),
+                    client->response.affected_rows TSRMLS_CC);
+            zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("insert_id"),
+                    client->response.insert_id TSRMLS_CC);
+
+            if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
+            {
+                zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
+                        ZEND_STRL("affected_rows"), client->response.affected_rows TSRMLS_CC);
+                zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
+                        ZEND_STRL("insert_id"), client->response.insert_id TSRMLS_CC);
+            }
+
             client->state = SW_MYSQL_STATE_QUERY;
 
             //OK
             if (client->response.response_type == 0)
             {
                 SW_ALLOC_INIT_ZVAL(result);
-                ZVAL_BOOL(result, 1);
+                if (client->cmd == SW_MYSQL_COM_STMT_PREPARE)
+                {
+                    if (client->statement_list == NULL)
+                    {
+                        client->statement_list = swLinkedList_new(0, NULL);
+                    }
+                    swLinkedList_append(client->statement_list, client->statement);
+                    object_init_ex(result, swoole_mysql_coro_statement_class_entry_ptr);
+                    swoole_set_object(result, client->statement);
+                    client->statement->object = sw_zval_dup(result);
+                }
+                else
+                {
+                    ZVAL_BOOL(result, 1);
+                }
             }
             //ERROR
             else if (client->response.response_type == 255)
@@ -1387,8 +1501,18 @@ static int swoole_mysql_coro_onRead(swReactor *reactor, swEvent *event)
                 SW_ALLOC_INIT_ZVAL(result);
                 ZVAL_BOOL(result, 0);
 
-                zend_update_property_stringl(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"), client->response.server_msg, client->response.l_server_msg TSRMLS_CC);
-                zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"), client->response.error_code TSRMLS_CC);
+                zend_update_property_stringl(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("error"),
+                        client->response.server_msg, client->response.l_server_msg TSRMLS_CC);
+                zend_update_property_long(swoole_mysql_coro_class_entry_ptr, zobject, ZEND_STRL("errno"),
+                        client->response.error_code TSRMLS_CC);
+
+                if (client->cmd == SW_MYSQL_COM_STMT_EXECUTE)
+                {
+                    zend_update_property_stringl(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
+                            ZEND_STRL("error"), client->response.server_msg, client->response.l_server_msg TSRMLS_CC);
+                    zend_update_property_long(swoole_mysql_coro_statement_class_entry_ptr, client->statement->object,
+                            ZEND_STRL("errno"), client->response.error_code TSRMLS_CC);
+                }
             }
             //ResultSet
             else
