@@ -22,8 +22,8 @@
 #endif
 #include "ext/standard/php_string.h"
 
-zend_class_entry swoole_redis_server_ce;
-zend_class_entry *swoole_redis_server_class_entry_ptr;
+static zend_class_entry swoole_redis_server_ce;
+static zend_class_entry *swoole_redis_server_class_entry_ptr;
 
 static swString *format_buffer;
 #ifdef SW_COROUTINE
@@ -68,6 +68,11 @@ void swoole_redis_server_init(int module_number TSRMLS_DC)
     swoole_redis_server_class_entry_ptr = sw_zend_register_internal_class_ex(&swoole_redis_server_ce, swoole_server_class_entry_ptr, "swoole_server" TSRMLS_CC);
     SWOOLE_CLASS_ALIAS(swoole_redis_server, "Swoole\\Redis\\Server");
 
+    if (SWOOLE_G(use_shortname))
+    {
+        zend_register_class_alias("Co\\Redis\\Server", swoole_redis_server_class_entry_ptr);
+    }
+
     zend_declare_class_constant_long(swoole_redis_server_class_entry_ptr, SW_STRL("NIL")-1, SW_REDIS_REPLY_NIL TSRMLS_CC);
     zend_declare_class_constant_long(swoole_redis_server_class_entry_ptr, SW_STRL("ERROR")-1, SW_REDIS_REPLY_ERROR TSRMLS_CC);
     zend_declare_class_constant_long(swoole_redis_server_class_entry_ptr, SW_STRL("STATUS")-1, SW_REDIS_REPLY_STATUS TSRMLS_CC);
